@@ -1,14 +1,14 @@
 
 /*
- *  agInit.c  $Id: agInit.c,v 4.11 2007/07/04 20:51:10 bkorb Exp $
+ *  agInit.c  $Id: agInit.c,v 4.22 2009/07/31 18:45:17 bkorb Exp $
  *
- *  Time-stamp:      "2007-07-04 11:14:11 bkorb"
+ *  Time-stamp:      "2009-07-31 11:06:50 bkorb"
  *
  *  Do all the initialization stuff.  For daemon mode, only
  *  children will return.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2007 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2009 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -25,7 +25,7 @@
  */
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by :mkfwd */
+/* static forward declarations maintained by mk-fwd */
 static void
 addSysEnv( char* pzEnvName );
 /* = = = END-STATIC-FORWARD = = = */
@@ -70,7 +70,7 @@ initialize( int arg_ct, char** arg_vec )
     SET_OPT_TEMPL_DIRS( "$$/../share/autogen" );
 
     {
-        char z[ 128 ] = "__autogen__";
+        char z[ SCRIBBLE_SIZE ] = "__autogen__";
 #if defined( HAVE_SOLARIS_SYSINFO )
         static const int nm[] = {
             SI_SYSNAME, SI_HOSTNAME, SI_ARCHITECTURE, SI_HW_PROVIDER,
@@ -116,8 +116,7 @@ initialize( int arg_ct, char** arg_vec )
 #endif
     }
 
-#ifndef DAEMON_ENABLED
-    doOptions( arg_ct, arg_vec );
+    doOptions(arg_ct, arg_vec);
 
     if (OPT_VALUE_TRACE > TRACE_NOTHING) {
         tSCC zBT[] = "(debug-enable 'backtrace)";
@@ -125,15 +124,7 @@ initialize( int arg_ct, char** arg_vec )
         ag_scm_c_eval_string_from_file_line( zBT, __FILE__, __LINE__ - 2 );
     }
 
-#else
-    optionSaveState( &autogenOptions );
-    doOptions( arg_ct, arg_vec );
-
-    if (OPT_VALUE_TRACE > TRACE_NOTHING) {
-        tSCC zBT[] = "(debug-enable 'backtrace)";
-        pzLastScheme = zBT;
-        ag_scm_c_eval_string_from_file_line( zBT, __FILE__, __LINE__ - 2 );
-    }
+#ifdef DAEMON_ENABLED
 
     if (! HAVE_OPT( DAEMON ))
         return;
@@ -168,9 +159,9 @@ addSysEnv( char* pzEnvName )
     int i = 2;
 
     for (;;) {
-        if (isupper( pzEnvName[i] ))
-            pzEnvName[i] = tolower( pzEnvName[i] );
-        else if (! isalnum( pzEnvName[i] ))
+        if (IS_UPPER_CASE_CHAR(pzEnvName[i]))
+            pzEnvName[i] = tolower(pzEnvName[i]);
+        else if (! IS_ALPHANUMERIC_CHAR(pzEnvName[i]))
             pzEnvName[i] = '_';
 
         if (pzEnvName[ ++i ] == NUL)
@@ -184,7 +175,7 @@ addSysEnv( char* pzEnvName )
     if (getenv( pzEnvName ) == NULL) {
         char* pz;
 
-        if (OPT_VALUE_TRACE > TRACE_NOTHING)
+        if (OPT_VALUE_TRACE > TRACE_DEBUG_MESSAGE)
             fprintf( pfTrace, "Adding ``%s'' to environment\n", pzEnvName );
         pz = aprf( zFmt, pzEnvName );
         TAGMEM( pz, "Added environment var" );
@@ -200,7 +191,7 @@ evalProto( tCC** ppzS, uint16_t* pProto )
 {
     tCC* pzS = *ppzS;
 
-    if (isalpha( *pzS )) {
+    if (IS_ALPHABETIC_CHAR(*pzS)) {
         inet_family_map_t* pMap = inet_family_map;
         do  {
             if (strncmp( pzS, pMap->pz_name, pMap->nm_len ) == 0) {
@@ -211,7 +202,7 @@ evalProto( tCC** ppzS, uint16_t* pProto )
         } while ( (++pMap)->pz_name != NULL );
     }
 
-    return isdigit( *pzS );
+    return IS_DEC_DIGIT_CHAR(*pzS);
 }
 
 
@@ -268,7 +259,8 @@ spawnPipe( tCC* pzFile )
         polls[0].events = POLLIN | POLLPRI;
 
         for (;;) {
-            int ct = poll( polls, 1, -1 );
+            char const * pzerr;
+            int   ct = poll( polls, 1, -1 );
             struct strrecvfd recvfd;
             pid_t child;
 
@@ -294,7 +286,8 @@ spawnPipe( tCC* pzFile )
                     continue;
 
                 case -1:
-                    AG_ABEND(aprf(zCannot, errno, "fork", zNil, strerror(errno)));
+                    pzerr = strerror(errno);
+                    AG_ABEND(aprf(zCannot, errno, "fork", zNil, pzerr));
 
                 case 0:
                 }
@@ -306,13 +299,16 @@ spawnPipe( tCC* pzFile )
                 recvfd.fd = fdpair.readFd;
 #endif /* ! HAVE_FATTACH */
 
-                if (dup2( recvfd.fd, STDIN_FILENO ) != STDIN_FILENO)
-                    AG_ABEND( aprf(zCannot, errno, "dup2", "stdin",
-                                   strerror(errno)));
+                if (dup2( recvfd.fd, STDIN_FILENO ) != STDIN_FILENO) {
+                    pzerr = strerror(errno);
+                    AG_ABEND(aprf(zCannot, errno, "dup2", "stdin", pzerr);
+                }
+
+                if (dup2( fdpair.writeFd, STDOUT_FILENO ) != STDOUT_FILENO) {
+                    pzerr = strerror(errno);
+                    AG_ABEND(aprf(zCannot, errno, "dup2", "stdout", pzerr);
+                }
 
-                if (dup2( fdpair.writeFd, STDOUT_FILENO ) != STDOUT_FILENO)
-                    AG_ABEND( aprf(zCannot, errno, "dup2", "stdout",
-                                   strerror(errno)));
                 return;
             }
         }
@@ -359,6 +355,7 @@ spawnPipe( tCC* pzFile )
             int ct = poll( polls, 1, -1 );
             struct strrecvfd recvfd;
             pid_t child;
+            char const * pzerr;
 
             switch (ct) {
             case -1:
@@ -382,7 +379,8 @@ spawnPipe( tCC* pzFile )
                     continue;
 
                 case -1:
-                    AG_ABEND(aprf(zCannot, errno, "fork", zNil, strerror(errno)));
+                    pzerr = strerror(errno);
+                    AG_ABEND(aprf(zCannot, errno, "fork", zNil, pzerr));
 
                 case 0:
                 }
@@ -398,13 +396,18 @@ spawnPipe( tCC* pzFile )
 
                 polls[0].fd = fdpair.writeFd;
                 polls[0].events = POLLOUT;
-                if (poll( polls, 1, -1 ) != 1)
+                if (poll( polls, 1, -1 ) != 1) {
+                    pzerr = strerror(errno);
                     AG_ABEND( aprf(zCannot, errno, "poll", "write pipe",
-                                   strerror( errno )));
+                                   pzerr));
+                }
 
-                if (dup2( fdpair.writeFd, STDOUT_FILENO ) != STDOUT_FILENO)
+                if (dup2( fdpair.writeFd, STDOUT_FILENO ) != STDOUT_FILENO) {
+                    pzerr = strerror(errno);
                     AG_ABEND( aprf(zCannot, errno, "dup2", pzOut,
-                                   strerror(errno)));
+                                   pzerr));
+                }
+
                 return;
             }
         }
@@ -474,7 +477,8 @@ spawnListens( tCC* pzPort, sa_family_t addr_family )
     break;
 
     default:
-        AG_ABEND(aprf("The '%d' address family cannot be handled", addr_family));
+        AG_ABEND(aprf("The '%d' address family cannot be handled",
+                      addr_family));
     }
 
     if (bind( socket_fd, &sa.addr, addr_len ) < 0) {
@@ -521,7 +525,9 @@ spawnListens( tCC* pzPort, sa_family_t addr_family )
         if (new_conns > 0) {
             switch (fork()) {
             default: continue;
-            case -1: AG_ABEND(aprf(zCannot, errno, "fork", zNil,strerror(errno)));
+            case -1:
+                AG_ABEND(aprf(zCannot, errno, "fork", zNil,strerror(errno)));
+
             case 0:  break;
             }
             break;
