@@ -1,16 +1,15 @@
 
 /*
  *  autogen.c
- *  $Id: autogen.c,v 4.27 2008/01/23 00:36:00 bkorb Exp $
+ *  $Id: autogen.c,v 4.33 2009/01/01 16:49:26 bkorb Exp $
  *
- *  Time-stamp:        "2007-12-02 12:06:21 bkorb"
- *  Last Committed:    $Date: 2008/01/23 00:36:00 $
+ *  Time-stamp:        "2008-08-17 15:30:54 bkorb"
+ *  Last Committed:    $Date: 2009/01/01 16:49:26 $
  *
  *  This is the main routine for autogen.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
- *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2009 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -63,6 +62,7 @@ signalSetup( sighandler_proc_t* chldHandler,
 static void
 inner_main( int argc, char** argv )
 {
+    atexit(doneCheck);
     initialize( argc, argv );
 
     procState = PROC_STATE_LOAD_DEFS;
@@ -86,6 +86,7 @@ inner_main( int argc, char** argv )
     }
 
     procState = PROC_STATE_DONE;
+    signalSetup( SIG_DFL, SIG_DFL );
     exit( EXIT_SUCCESS );
 }
 
@@ -103,7 +104,7 @@ main(int argc, char** argv)
     if (sigsetjmp( abendJumpEnv, 0 ) != 0)
         signalExit( abendJumpSignal );
 
-    signalSetup( ignoreSignal, abendSignal );
+    signalSetup(ignoreSignal, abendSignal);
 
 #if GUILE_VERSION >= 107000
     if (getenv( "GUILE_WARN_DEPRECATED" ) == NULL)
@@ -183,8 +184,15 @@ signalExit( int sig )
 static void
 abendSignal( int sig )
 {
-    abendJumpSignal = sig;
-    siglongjmp( abendJumpEnv, sig );
+    switch (procState) {
+    case PROC_STATE_DONE:
+    case PROC_STATE_ABORTING:
+        break;
+
+    default:
+        abendJumpSignal = sig;
+        siglongjmp( abendJumpEnv, sig );
+    }
 }
 
 
@@ -217,6 +225,16 @@ doneCheck( void )
     int exit_code = EXIT_SUCCESS;
 #endif
 
+    /*
+     *  There are contexts wherein this function can get registered twice.
+     */
+    {
+        static int doneCheckDone = 0;
+        if (doneCheckDone)
+            return;
+        doneCheckDone = 1;
+    }
+
 #ifdef SHELL_ENABLED
     ag_scm_c_eval_string_from_file_line(
         "(if (> (string-length shell-cleanup) 0)"
@@ -395,7 +413,6 @@ signalSetup( sighandler_proc_t* chldHandler,
 #else
     const int maxSig = NSIG;
 #endif
-    atexit( doneCheck );
 
     sa.sa_flags   = 0;
     sigemptyset( &sa.sa_mask );
