@@ -1,16 +1,15 @@
 
 /*
- *  $Id: defLex.c,v 4.27 2008/01/23 00:36:00 bkorb Exp $
+ *  $Id: defLex.c,v 4.29 2009/01/01 16:49:26 bkorb Exp $
  *
- *  Time-stamp:        "2007-11-12 22:33:12 bkorb"
- *  Last Committed:    $Date: 2008/01/23 00:36:00 $
+ *  Time-stamp:        "2008-07-26 10:06:56 bkorb"
+ *  Last Committed:    $Date: 2009/01/01 16:49:26 $
  *
  *  This module scans the template variable declarations and passes
  *  tokens back to the parser.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
- *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2009 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -164,7 +163,7 @@ scanAgain:
         if (pCurCtx->pzScan[1] != '<')
             goto BrokenToken;
 
-        pz = assembleHereString( pCurCtx->pzScan + 2 );
+        pz = assembleHereString(pCurCtx->pzScan + 2);
         if (pz == NULL) {
             lastToken = DP_EV_INVALID;
             return DP_EV_INVALID;
@@ -207,7 +206,8 @@ scanAgain:
 
     case '`':
     {
-        char* pz = ao_string_cook( pCurCtx->pzScan, &(pCurCtx->lineNo));
+        int line_no = pCurCtx->lineNo;
+        char* pz = ao_string_cook( pCurCtx->pzScan, &line_no);
 
         if (pz == NULL)
             goto NUL_error;
@@ -218,6 +218,8 @@ scanAgain:
 
         lastToken = DP_EV_STRING;
         pz = runShell( (char const*)pz_token );
+        pCurCtx->lineNo = line_no;
+
         if (pz == NULL)
             goto scanAgain;
         TAGMEM( pz, "shell definition string" );
@@ -333,13 +335,21 @@ loadScheme( void )
     char*    pzEnd     = (char*)skipScheme( pzText, pzText + strlen( pzText ));
     char     endCh     = *pzEnd;
     int      schemeLen = (pzEnd - pzText);
+    int      next_ln;
     SCM      res;
 
     /*
      *  NUL terminate the Scheme expression, run it, then restore
      *  the NUL-ed character.
      */
-    *pzEnd = NUL;
+    if (*pzEnd == NUL)
+        AG_ABEND(aprf(zErrMsg, pzProg,
+                      "end of Guile/scheme expression not found",
+                      pCurCtx->pzCtxFname, pCurCtx->lineNo));
+
+    *pzEnd  = NUL;
+    next_ln = pCurCtx->lineNo + count_nl(pzText);
+
     procState = PROC_STATE_GUILE_PRELOAD;
     res = ag_scm_c_eval_string_from_file_line(
         pzText, pCurCtx->pzCtxFname, pCurCtx->lineNo );
@@ -348,6 +358,8 @@ loadScheme( void )
 
     pCurCtx->pzScan = pzEnd;
     pzEnd = (char*)resolveSCM( res ); /* ignore const-ness */
+    pCurCtx->lineNo = next_ln;
+
     if (strlen( pzEnd ) >= schemeLen) {
         AGDUPSTR( pzEnd, pzEnd, "SCM Result" );
 
@@ -519,10 +531,12 @@ assembleName( char* pzScan, te_dp_event* pRetVal )
 static char*
 assembleHereString( char* pzScan )
 {
+    static char const endless[] = "Unterminated HereString";
     ag_bool  trimTabs = AG_FALSE;
     char     zMark[ MAX_HEREMARK_LEN ];
     size_t   markLen = 0;
     char*    pzDest;
+    int      here_string_line_no;
 
     /*
      *  See if we are to strip leading tab chars
@@ -568,13 +582,13 @@ assembleHereString( char* pzScan )
      */
     pzScan = strchr( pzScan, '\n' );
     if (pzScan == NULL)
-        AG_ABEND( aprf( zErrMsg, pzProg, "Unterminated HereString",
-                        pCurCtx->pzCtxFname, pCurCtx->lineNo ));
+        AG_ABEND( aprf( zErrMsg, pzProg, endless, pCurCtx->pzCtxFname,
+                        pCurCtx->lineNo ));
 
     /*
      *  And skip the first new line + conditionally skip tabs
      */
-    pCurCtx->lineNo++;
+    here_string_line_no = pCurCtx->lineNo++;
     pzScan++;
 
     if (trimTabs)
@@ -595,8 +609,8 @@ assembleHereString( char* pzScan )
                 goto lineDone;
 
             case NUL:
-                AG_ABEND( aprf( zErrMsg, pzProg, "Unterminated HereString",
-                                pCurCtx->pzCtxFname, pCurCtx->lineNo ));
+                AG_ABEND( aprf( zErrMsg, pzProg, endless, pCurCtx->pzCtxFname,
+                                here_string_line_no));
             }
         } lineDone:;
 
