@@ -1,10 +1,10 @@
 /*
  *  expFormat.c
  *
- *  Time-stamp:        "2007-11-19 21:12:49 bkorb"
- *  Last Committed:    $Date: 2009/01/01 16:51:42 $
+ *  Time-stamp:        "2009-07-31 12:05:29 bkorb"
+ *  Last Committed:    $Date: 2009/07/31 19:10:07 $
  *
- *  $Id: expFormat.c,v 4.27 2009/01/01 16:51:42 bkorb Exp $
+ *  $Id: expFormat.c,v 4.27 2009/07/31 19:10:07 bkorb Exp $
  *  This module implements formatting expression functions.
  *
  *  This file is part of AutoGen.
@@ -93,14 +93,14 @@ static char const zDne1[] =
 static char const zDne[] = "%6$s"
 "%1$sDO NOT EDIT THIS FILE   (%2$s)\n"
 "%1$s\n"
-"%1$sIt has been AutoGen-ed  %3$s\n"
+"%1$sIt has been AutoGen-ed%3$s\n"
 "%1$sFrom the definitions    %4$s\n"
 "%1$sand the template file   %5$s";
 
 static char const zDne2[] = "%6$s"
 "%1$sEDIT THIS FILE WITH CAUTION  (%2$s)\n"
 "%1$s\n"
-"%1$sIt has been AutoGen-ed  %3$s\n"
+"%1$sIt has been AutoGen-ed%3$s\n"
 "%1$sFrom the definitions    %4$s\n"
 "%1$sand the template file   %5$s";
 
@@ -121,7 +121,14 @@ tSCC zFmtAlloc[] = "asprintf allocation";
  *
  * doc:  Generate a "DO NOT EDIT" or "EDIT WITH CARE" warning string.
  *       Which depends on whether or not the @code{--writable} command line
- *       option was set.  The first argument is a per-line string prefix.
+ *       option was set.
+ *
+ *       The first argument may be an option:  -d
+ *       This will suppress the variable text (date and version information).
+ *       If specified, then the "prefix" and "first" arguments are shifted
+ *       to the next arguments.
+ *
+ *       The first argument is a per-line string prefix.
  *       The optional second argument is a prefix for the first-line and,
  *       in read-only mode, activates the editor hints.
  *       @*
@@ -141,7 +148,7 @@ SCM
 ag_scm_dne( SCM prefix, SCM first, SCM opt )
 {
     int      noDate   = 0;
-    char     zScribble[ 128 ];
+    char     zScribble[ SCRIBBLE_SIZE ];
     char*    pzRes;
     tCC*     pzFirst  = zNil;
     SCM      res;
@@ -158,8 +165,8 @@ ag_scm_dne( SCM prefix, SCM first, SCM opt )
         first    = opt;
     }
 
-    if (pfxLen > 128 )
-        AG_ABEND( aprf( zPfxMsg, zPfxLen, 128 ));
+    if (pfxLen > SCRIBBLE_SIZE )
+        AG_ABEND( aprf( zPfxMsg, zPfxLen, SCRIBBLE_SIZE ));
 
     /*
      *  IF we also have a 'first' prefix string,
@@ -167,8 +174,8 @@ ag_scm_dne( SCM prefix, SCM first, SCM opt )
      */
     if (AG_SCM_STRING_P( first )) {
         int len = AG_SCM_STRLEN( first );
-        if (len >= 128)
-            AG_ABEND( aprf( zPfxMsg, zPfxLen, 128 ));
+        if (len >= SCRIBBLE_SIZE)
+            AG_ABEND( aprf( zPfxMsg, zPfxLen, SCRIBBLE_SIZE ));
 
         pzFirst = aprf( ENABLED_OPT( WRITABLE ) ? "%s\n" : zDne1,
                         ag_scm2zchars( first, "first-prefix" ), pzPrefix );
@@ -177,13 +184,18 @@ ag_scm_dne( SCM prefix, SCM first, SCM opt )
     if (noDate) {
         zScribble[0] = NUL;
     } else {
+        static char const tim_fmt[] =
+            "  %B %e, %Y at %r by AutoGen " AUTOGEN_VERSION;
         time_t    curTime = time( NULL );
         struct tm*  pTime = localtime( &curTime );
-        strftime( zScribble, (size_t)128, "%A %B %e, %Y at %r %Z", pTime );
+        strftime(zScribble, (size_t)SCRIBBLE_SIZE, tim_fmt, pTime);
     }
 
     {
         char const* pz;
+        tFpStack*   pfp = pCurFp;
+
+        while (pfp->flags & FPF_UNLINK)  pfp = pfp->pPrev;
         if (! ENABLED_OPT( DEFINITIONS ))
             pz = "<<no definitions>>";
 
@@ -196,9 +208,9 @@ ag_scm_dne( SCM prefix, SCM first, SCM opt )
                 pz = "stdin";
         }
 
-        pzRes = aprf( ENABLED_OPT( WRITABLE ) ? zDne2 : zDne,
-                      pzPrefix, pCurFp->pzOutName, zScribble,
-                      pz, pzTemplFileName, pzFirst );
+        pzRes = aprf(ENABLED_OPT( WRITABLE ) ? zDne2 : zDne,
+                     pzPrefix, pfp->pzOutName, zScribble,
+                     pz, pzTemplFileName, pzFirst);
     }
 
     if (pzRes == NULL)
@@ -344,15 +356,15 @@ ag_scm_gpl( SCM prog_name, SCM prefix )
     SCM     res;
 
     /*
-     *  Get the addresses of the program name and prefix strings.
-     *  Make sure they are reasonably sized (<256 for program name
-     *  and <128 for a line prefix).  Copy them to the scratch buffer.
+     *  Get the addresses of the program name prefix and owner strings.
+     *  Make sure they are reasonably sized (less than
+     *  SCRIBBLE_SIZE).  Copy them to the scratch buffer.
      */
-    if (AG_SCM_STRLEN( prog_name ) >= 256)
-        AG_ABEND( aprf( zPfxMsg, zProgLen, 256 ));
+    if (AG_SCM_STRLEN( prog_name ) >= SCRIBBLE_SIZE)
+        AG_ABEND( aprf( zPfxMsg, zProgLen, SCRIBBLE_SIZE ));
 
-    if (AG_SCM_STRLEN( prefix ) >= 128)
-        AG_ABEND( aprf( zPfxMsg, zPfxLen, 128 ));
+    if (AG_SCM_STRLEN( prefix ) >= SCRIBBLE_SIZE)
+        AG_ABEND( aprf( zPfxMsg, zPfxLen, SCRIBBLE_SIZE ));
 
     /*
      *  Allocate-sprintf the result string, then put it in a new SCM.
@@ -390,15 +402,15 @@ ag_scm_agpl( SCM prog_name, SCM prefix )
     SCM     res;
 
     /*
-     *  Get the addresses of the program name and prefix strings.
-     *  Make sure they are reasonably sized (<256 for program name
-     *  and <128 for a line prefix).  Copy them to the scratch buffer.
+     *  Get the addresses of the program name prefix and owner strings.
+     *  Make sure they are reasonably sized (less than
+     *  SCRIBBLE_SIZE).  Copy them to the scratch buffer.
      */
-    if (AG_SCM_STRLEN( prog_name ) >= 256)
-        AG_ABEND( aprf( zPfxMsg, zProgLen, 256 ));
+    if (AG_SCM_STRLEN( prog_name ) >= SCRIBBLE_SIZE)
+        AG_ABEND( aprf( zPfxMsg, zProgLen, SCRIBBLE_SIZE ));
 
-    if (AG_SCM_STRLEN( prefix ) >= 128)
-        AG_ABEND( aprf( zPfxMsg, zPfxLen, 128 ));
+    if (AG_SCM_STRLEN( prefix ) >= SCRIBBLE_SIZE)
+        AG_ABEND( aprf( zPfxMsg, zPfxLen, SCRIBBLE_SIZE ));
 
     /*
      *  Allocate-sprintf the result string, then put it in a new SCM.
@@ -438,17 +450,17 @@ ag_scm_lgpl( SCM prog_name, SCM owner, SCM prefix )
 
     /*
      *  Get the addresses of the program name prefix and owner strings.
-     *  Make sure they are reasonably sized (<256 for program name
-     *  and <128 for a line prefix).  Copy them to the scratch buffer.
+     *  Make sure they are reasonably sized (less than
+     *  SCRIBBLE_SIZE).  Copy them to the scratch buffer.
      */
-    if (AG_SCM_STRLEN( prog_name ) >= 256)
-        AG_ABEND( aprf( zPfxMsg, zProgLen, 256 ));
+    if (AG_SCM_STRLEN( prog_name ) >= SCRIBBLE_SIZE)
+        AG_ABEND( aprf( zPfxMsg, zProgLen, SCRIBBLE_SIZE ));
 
-    if (AG_SCM_STRLEN( prefix ) >= 128)
-        AG_ABEND( aprf( zPfxMsg, zPfxLen, 128 ));
+    if (AG_SCM_STRLEN( prefix ) >= SCRIBBLE_SIZE)
+        AG_ABEND( aprf( zPfxMsg, zPfxLen, SCRIBBLE_SIZE ));
 
-    if (AG_SCM_STRLEN( owner ) >= 256)
-        AG_ABEND( aprf( zPfxMsg, zOwnLen, 256 ));
+    if (AG_SCM_STRLEN( owner ) >= SCRIBBLE_SIZE)
+        AG_ABEND( aprf( zPfxMsg, zOwnLen, SCRIBBLE_SIZE ));
 
     /*
      *  Allocate-sprintf the result string, then put it in a new SCM.
@@ -495,17 +507,17 @@ ag_scm_bsd( SCM prog_name, SCM owner, SCM prefix )
 
     /*
      *  Get the addresses of the program name prefix and owner strings.
-     *  Make sure they are reasonably sized (<256 for program name
-     *  and <128 for a line prefix).  Copy them to the scratch buffer.
+     *  Make sure they are reasonably sized (less than
+     *  SCRIBBLE_SIZE).  Copy them to the scratch buffer.
      */
-    if (AG_SCM_STRLEN( prog_name ) >= 256)
-        AG_ABEND( aprf( zPfxMsg, zProgLen, 256 ));
+    if (AG_SCM_STRLEN( prog_name ) >= SCRIBBLE_SIZE)
+        AG_ABEND( aprf( zPfxMsg, zProgLen, SCRIBBLE_SIZE ));
 
-    if (AG_SCM_STRLEN( prefix ) >= 128)
-        AG_ABEND( aprf( zPfxMsg, zPfxLen, 128 ));
+    if (AG_SCM_STRLEN( prefix ) >= SCRIBBLE_SIZE)
+        AG_ABEND( aprf( zPfxMsg, zPfxLen, SCRIBBLE_SIZE ));
 
-    if (AG_SCM_STRLEN( owner ) >= 256)
-        AG_ABEND( aprf( zPfxMsg, zOwnLen, 256 ));
+    if (AG_SCM_STRLEN( owner ) >= SCRIBBLE_SIZE)
+        AG_ABEND( aprf( zPfxMsg, zOwnLen, SCRIBBLE_SIZE ));
 
     /*
      *  Allocate-sprintf the result string, then put it in a new SCM.
@@ -591,17 +603,17 @@ ag_scm_license( SCM license, SCM prog_name, SCM owner, SCM prefix )
 
     /*
      *  Get the addresses of the program name prefix and owner strings.
-     *  Make sure they are reasonably sized (<256 for program name
-     *  and <128 for a line prefix).  Copy them to the scratch buffer.
+     *  Make sure they are reasonably sized (less than
+     *  SCRIBBLE_SIZE).  Copy them to the scratch buffer.
      */
-    if (AG_SCM_STRLEN( prog_name ) >= 256)
-        AG_ABEND( aprf( zPfxMsg, zProgLen, 256 ));
+    if (AG_SCM_STRLEN( prog_name ) >= SCRIBBLE_SIZE)
+        AG_ABEND( aprf( zPfxMsg, zProgLen, SCRIBBLE_SIZE ));
 
-    if (AG_SCM_STRLEN( prefix ) >= 128)
-        AG_ABEND( aprf( zPfxMsg, zPfxLen, 128 ));
+    if (AG_SCM_STRLEN( prefix ) >= SCRIBBLE_SIZE)
+        AG_ABEND( aprf( zPfxMsg, zPfxLen, SCRIBBLE_SIZE ));
 
-    if (AG_SCM_STRLEN( owner ) >= 256)
-        AG_ABEND( aprf( zPfxMsg, zOwnLen, 256 ));
+    if (AG_SCM_STRLEN( owner ) >= SCRIBBLE_SIZE)
+        AG_ABEND( aprf( zPfxMsg, zOwnLen, SCRIBBLE_SIZE ));
 
     /*
      *  Reformat the string with the given arguments
