@@ -1,42 +1,44 @@
 
 /*
  *  expString.c
- *  $Id: expString.c,v 4.17 2007/04/28 22:19:23 bkorb Exp $
+ *  $Id: expString.c,v 4.24 2009/07/21 03:21:57 bkorb Exp $
  *
- *  Time-stamp:        "2006-12-10 11:05:08 bkorb"
- *  Last Committed:    $Date: 2007/04/28 22:19:23 $
+ *  Time-stamp:        "2009-07-09 19:06:44 bkorb"
+ *  Last Committed:    $Date: 2009/07/21 03:21:57 $
  *
  *  This module implements expression functions that
  *  manipulate string values.
- */
-
-/*
- *  AutoGen copyright 1992-2007 Bruce Korb
  *
- *  AutoGen is free software.
- *  You may redistribute it and/or modify it under the terms of the
- *  GNU General Public License, as published by the Free Software
- *  Foundation; either version 2, or (at your option) any later version.
+ *  This file is part of AutoGen.
+ *  AutoGen copyright (c) 1992-2009 by Bruce Korb - all rights reserved
  *
- *  AutoGen is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *  See the GNU General Public License for more details.
+ * AutoGen is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with AutoGen.  See the file "COPYING".  If not,
- *  write to:  The Free Software Foundation, Inc.,
- *             51 Franklin Street, Fifth Floor,
- *             Boston, MA  02110-1301, USA.
+ * AutoGen is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by :mkfwd */
+/* static forward declarations maintained by mk-fwd */
+static size_t
+string_size(char const * pzScn, size_t newLineSize);
+
 static SCM
 makeString( tCC*    pzText,
             tCC*    pzNewLine,
             size_t  newLineSize );
 
+static size_t
+stringify_for_shell(char * pzNew, u_int qt, char const * pzDta, size_t dtaSize);
+
 static SCM
 shell_stringify( SCM obj, u_int qt );
 
@@ -57,20 +59,13 @@ do_multi_subs(
     SCM         repl );
 /* = = = END-STATIC-FORWARD = = = */
 
-static SCM
-makeString( tCC*    pzText,
-            tCC*    pzNewLine,
-            size_t  newLineSize )
+static size_t
+string_size(char const * pzScn, size_t newLineSize)
 {
-    char     z[ 256 ];
-    char*    pzDta;
-    char*    pzFre;
-    tCC*     pzScn   = pzText;
-
     /*
      *  Start by counting the start and end quotes, plus the NUL.
      */
-    size_t   dtaSize = sizeof( "\"\"" );
+    size_t dtaSize = sizeof( "\"\"" );
 
     for (;;) {
         char ch = *(pzScn++);
@@ -90,7 +85,7 @@ makeString( tCC*    pzText,
          */
         else switch (ch) {
         case NUL:
-            goto loopBreak;
+            return dtaSize;
 
         case '\n':
             dtaSize += newLineSize;
@@ -108,7 +103,19 @@ makeString( tCC*    pzText,
         default:
             dtaSize += 4;
         }
-    } loopBreak:;
+    }
+}
+
+static SCM
+makeString( tCC*    pzText,
+            tCC*    pzNewLine,
+            size_t  newLineSize )
+{
+    char     z[ SCRIBBLE_SIZE ];
+    char*    pzDta;
+    char*    pzFre;
+    tCC*     pzScn   = pzText;
+    size_t   dtaSize = string_size(pzText, newLineSize);
 
     /*
      *  We now know how big the string is going to be.
@@ -118,8 +125,6 @@ makeString( tCC*    pzText,
          pzFre = pzDta = AGALOC( dtaSize, "quoting string" );
     else pzFre = pzDta = z;
 
-    pzScn = pzText;
-
     *(pzDta++) = '"';
 
     for (;;) {
@@ -224,97 +229,94 @@ makeString( tCC*    pzText,
     }
 }
 
+static size_t
+stringify_for_shell(char * pzNew, u_int qt, char const * pzDta, size_t dtaSize)
+{
+    char * pz = pzNew;
+    *(pz++) = qt;
+
+    for (;;) {
+        char c = (*(pz++) = *(pzDta++));
+        switch (c) {
+        case NUL:
+            pz[-1]  = qt;
+            *pz     = NUL;
+            dtaSize = (pz - pzNew);
+
+            return (pz - pzNew);
+
+        case '\\':
+            /*
+             *  If someone went to the trouble to escape a backquote or a
+             *  dollar sign, then we should not neutralize it.  Note that
+             *  we handle a following backslash as a normal character.
+             *
+             *  i.e.  \\ --> \\\\ *BUT* \\$ --> \\\$
+             */
+            c = *pzDta;
+            switch (*pzDta) {
+            case '$':
+                break;
+
+            case '"':
+            case '`':
+                /*
+                 *  IF the ensuing quote character does *NOT* match the
+                 *  quote character for the string, then we will preserve
+                 *  the single copy of the backslash.  If it does match,
+                 *  then we will double the backslash and a third backslash
+                 *  will be inserted when we emit the quote character.
+                 */
+                if (c != qt)
+                    break;
+                /* FALLTHROUGH */
+
+            default:
+                *(pz++) = '\\';   /* \   -->  \\    */
+            }
+            break;
+
+        case '"': case '`':
+            if (c == qt) {
+                /*
+                 *  This routine does both `xx` and "xx" strings, we have
+                 *  to worry about this stuff differently.  I.e., in ""
+                 *  strings, add a single \ in front of ", and in ``
+                 *  preserve a add \ in front of `.
+                 */
+                pz[-1]  = '\\';       /* "   -->   \"   */
+                *(pz++) = c;
+            }
+        }
+    }
+}
 
 static SCM
 shell_stringify( SCM obj, u_int qt )
 {
-    char*  pzDta;
     char*  pzNew;
     size_t dtaSize = 3;
+    char*  pzDta   = ag_scm2zchars( obj, "AG Object" );
+    char*  pz      = pzDta;
 
-    pzDta = ag_scm2zchars( obj, "AG Object" );
+    for (;;) {
+        char c = *(pz++);
 
-    {
-        char*  pz = pzDta;
+        switch (c) {
+        case NUL:
+            goto loopDone1;
 
-        for (;;) {
-            char c = *(pz++);
+        case '"': case '`': case '\\':
+            dtaSize += 2;
+            break;
 
-            switch (c) {
-            case NUL:
-                goto loopDone1;
-
-            case '"': case '`': case '\\':
-                dtaSize += 2;
-                break;
-
-            default:
-                dtaSize++;
-            }
-        } loopDone1:;
-    }
+        default:
+            dtaSize++;
+        }
+    } loopDone1:;
 
     pzNew = AGALOC( dtaSize, "shell string" );
-
-    {
-        char* pz = pzNew;
-        *(pz++) = qt;
-
-        for (;;) {
-            char c = (*(pz++) = *(pzDta++));
-            switch (c) {
-            case NUL:
-                goto loopDone2;
-
-            case '\\':
-                /*
-                 *  If someone went to the trouble to escape a backquote or a
-                 *  dollar sign, then we should not neutralize it.  Note that
-                 *  we handle a following backslash as a normal character.
-                 *
-                 *  i.e.  \\ --> \\\\ *BUT* \\$ --> \\\$
-                 */
-                c = *pzDta;
-                switch (*pzDta) {
-                case '$':
-                    break;
-
-                case '"':
-                case '`':
-                    /*
-                     *  IF the ensuing quote character does *NOT* match the
-                     *  quote character for the string, then we will preserve
-                     *  the single copy of the backslash.  If it does match,
-                     *  then we will double the backslash and a third backslash
-                     *  will be inserted when we emit the quote character.
-                     */
-                    if (c != qt)
-                        break;
-                    /* FALLTHROUGH */
-
-                default:
-                    *(pz++) = '\\';   /* \   -->  \\    */
-                }
-                break;
-
-            case '"': case '`':
-                if (c == qt) {
-                    /*
-                     *  This routine does both `xx` and "xx" strings, we have
-                     *  to worry about this stuff differently.  I.e., in ""
-                     *  strings, add a single \ in front of ", and in ``
-                     *  preserve a add \ in front of `.
-                     */
-                    pz[-1]  = '\\';       /* "   -->   \"   */
-                    *(pz++) = c;
-                }
-            }
-        } loopDone2:;
-
-        pz[-1]  = qt;
-        *pz     = NUL;
-        dtaSize = (pz - pzNew);
-    }
+    dtaSize = stringify_for_shell(pzNew, qt, pzDta, dtaSize);
 
     {
         SCM res = AG_SCM_STR2SCM( pzNew, dtaSize );
@@ -960,11 +962,11 @@ ag_scm_sub_shell_str( SCM obj )
 SCM
 ag_scm_stack( SCM obj )
 {
-    SCM   res;
-    SCM * pos = &res;
+    SCM         res;
+    SCM *       pos = &res;
     tDefEntry** ppDE;
-        tDefEntry* pDE;
-        SCM        str;
+    tDefEntry*  pDE;
+    SCM         str;
 
     res = SCM_EOL;
 
@@ -1063,55 +1065,53 @@ ag_scm_c_string( SCM str )
 SCM
 ag_scm_string_tr_x( SCM str, SCM from_xform, SCM to_xform )
 {
-    {
-        char  map[ 256 ];
-        int   i = 255;
-        char* pzFrom = ag_scm2zchars( from_xform, "string" );
-        char* pzTo   = ag_scm2zchars(   to_xform, "string" );
+    unsigned char map[ 1 << 8 /* bits-per-byte */ ];
+    int   i      = sizeof(map) - 1;
+    char* pzFrom = ag_scm2zchars( from_xform, "string" );
+    char* pzTo   = ag_scm2zchars(   to_xform, "string" );
 
-        do  {
-            map[i] = i;
-        } while (--i > 0);
+    do  {
+        map[i] = i;
+    } while (--i > 0);
 
-        for (;i <= 255;i++) {
-            unsigned char fch = (unsigned char)*(pzFrom++);
-            unsigned char tch = (unsigned char)*(pzTo++);
+    for (;i <= sizeof(map) - 1;i++) {
+        unsigned char fch = (unsigned char)*(pzFrom++);
+        unsigned char tch = (unsigned char)*(pzTo++);
 
-            if (tch == NUL) {
-                pzTo--;
-                tch = pzTo[-1];
-            }
-
-            switch (fch) {
-            case NUL:
-                goto mapDone;
-
-            case '-':
-                if ((i > 0) && (tch == '-')) {
-                    unsigned char fs = (unsigned char)pzFrom[-2];
-                    unsigned char fe = (unsigned char)pzFrom[0];
-                    unsigned char ts = (unsigned char)pzTo[-2];
-                    unsigned char te = (unsigned char)pzTo[0];
-                    if (te != NUL) {
-                        while (fs < fe) {
-                            map[ fs++ ] = ts;
-                            if (ts < te) ts++;
-                        }
-                        break;
-                    }
-                }
-
-            default:
-                map[ fch ] = tch;
-            }
-        } mapDone:;
-
-        pzTo = (char*)(void*)AG_SCM_CHARS( str );
-        i    = AG_SCM_STRLEN( str );
-        while (i-- > 0) {
-            *pzTo = map[ (int)*pzTo ];
-            pzTo++;
+        if (tch == NUL) {
+            pzTo--;
+            tch = pzTo[-1];
         }
+
+        switch (fch) {
+        case NUL:
+            goto mapDone;
+
+        case '-':
+            if ((i > 0) && (tch == '-')) {
+                unsigned char fs = (unsigned char)pzFrom[-2];
+                unsigned char fe = (unsigned char)pzFrom[0];
+                unsigned char ts = (unsigned char)pzTo[-2];
+                unsigned char te = (unsigned char)pzTo[0];
+                if (te != NUL) {
+                    while (fs < fe) {
+                        map[ fs++ ] = ts;
+                        if (ts < te) ts++;
+                    }
+                    break;
+                }
+            }
+
+        default:
+            map[ fch ] = tch;
+        }
+    } mapDone:;
+
+    pzTo = (char*)(void*)AG_SCM_CHARS( str );
+    i    = AG_SCM_STRLEN( str );
+    while (i-- > 0) {
+        *pzTo = map[ (int)*pzTo ];
+        pzTo++;
     }
     return str;
 }
@@ -1188,6 +1188,37 @@ ag_scm_string_substitute( SCM Str, SCM Match, SCM Repl )
     return res;
 }
 
+
+/*=gfunc time_string_to_number
+ *
+ * what:   duration string to seconds
+ * general_use:
+ * exparg: time_spec, string to parse
+ *
+ * doc:    Convert the argument string to a time period in seconds.
+ *         The string may use multiple parts consisting of days, hours
+ *         minutes and seconds.  These are indicated with a suffix of
+ *         @code{d}, @code{h}, @code{m} and @code{s} respectively.
+ *         Hours, minutes and seconds may also be represented with
+ *         @code{HH:MM:SS} or, without hours, as @code{MM:SS}.
+=*/
+SCM
+ag_scm_time_string_to_number( SCM time_spec )
+{
+    extern time_t parse_duration(char const * in_pz);
+
+    tCC *   pz;
+    time_t  time_period;
+
+    if (! AG_SCM_STRING_P( time_spec ))
+        return SCM_UNDEFINED;
+
+    pz = AG_SCM_CHARS( time_spec );
+    time_period = parse_duration(pz);
+    
+    return AG_SCM_INT2SCM((int)time_period);
+}
+
 /*
  * Local Variables:
  * mode: C
